package cn.com.qmth.stmms.biz.exam.service.impl;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.dao.MarkerDao;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.query.MarkerSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.mark.dao.MarkLibraryDao;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import cn.com.qmth.stmms.common.enums.MarkStatus;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

@Service("markerService")
public class MarkerServiceImpl extends BaseQueryService<Marker> implements MarkerService {

    @Autowired
    private MarkerDao markerDao;

    @Autowired
    private MarkLibraryDao libraryDao;

    @Autowired
    private MarkGroupService groupService;

    @Transactional
    @Override
    public Marker save(Marker marker) {
        return markerDao.save(marker);
    }

    @Override
    public Marker findById(Integer id) {
        return markerDao.findOne(id);
    }

    @Override
    public List<Marker> findByExamAndSubjectAndGroup(int examId, String subjectCode, int number) {
        return markerDao.findByExamIdAndSubjectCodeAndGroupNumber(examId, subjectCode, number, null);
    }

    @Override
    public long countByExam(int examId) {
        return markerDao.countByExamId(examId);
    }

    @Override
    public long countByExamAndSubjectAndGroup(int examId, String subjectCode, int number) {
        return markerDao.countByExamIdAndSubjectCodeAndGroupNumber(examId, subjectCode, number);
    }

    @Override
    public List<Marker> getMarkCount(int examId) {
        List<Marker> list = new LinkedList<Marker>();
        List<Object[]> result = libraryDao.countMarkerAndStatus(examId, LibraryStatus.MARKED, LibraryStatus.INSPECTED);
        if (result != null) {
            for (Object[] array : result) {
                try {
                    Marker marker = findById((int) array[0]);
                    marker.setMarkedCount((long) array[1]);
                    list.add(marker);
                } catch (Exception e) {
                }
            }
        }
        return list;
    }

    @Override
    public List<Marker> getMarkCount(int examId, Set<String> subjectCodes) {
        List<Marker> list = new LinkedList<Marker>();
        List<Object[]> result = libraryDao.countMarkerAndStatusAndSubjectCodeIn(examId, subjectCodes,
                LibraryStatus.MARKED, LibraryStatus.INSPECTED);
        if (result != null) {
            for (Object[] array : result) {
                try {
                    Marker marker = findById((int) array[0]);
                    marker.setMarkedCount((long) array[1]);
                    list.add(marker);
                } catch (Exception e) {
                }
            }
        }
        return list;
    }

    @Override
    public MarkerSearchQuery findByQuery(final MarkerSearchQuery query) {
        checkQuery(query);
        Page<Marker> result = markerDao.findAll(buildSpecification(query), query);
        fillResult(result, query);
        return query;
    }

    private Specification<Marker> buildSpecification(final MarkerSearchQuery query) {
        return new Specification<Marker>() {

            @Override
            public Predicate toPredicate(Root<Marker> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();
                if (query.getId() != null) {
                    predicates.add(cb.equal(root.get("id"), query.getId()));
                }
                if (query.getExamId() != null) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (StringUtils.isNotBlank(query.getSubjectCode())) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                if (query.getGroupNumber() != null) {
                    predicates.add(cb.equal(root.get("groupNumber"), query.getGroupNumber()));
                }
                if (query.getUserId() != null) {
                    predicates.add(cb.equal(root.get("userId"), query.getUserId()));
                }
                return predicates.isEmpty() ? cb.conjunction() : cb.and(predicates.toArray(new Predicate[predicates
                        .size()]));
            }
        };
    }

    @Override
    public int batchSave(List<Marker> list) {
        markerDao.save(list);
        return list.size();
    }

    @Override
    public Marker findByExamAndSubjectAndNumberAndUserId(int examId, String subjectCode, Integer number, Integer userId) {
        return markerDao.findByExamIdAndSubjectCodeAndGroupNumberAndUserId(examId, subjectCode, number, userId);
    }

    @Override
    public List<Marker> findByExamAndUserId(Integer examId, Integer userId) {
        return markerDao.findByExamIdAndUserIdAndGroupStatus(examId, userId, MarkStatus.TRIAL, MarkStatus.FORMAL);
    }

    @Override
    public long countByUserId(Integer userId) {
        return markerDao.countByUserId(userId);
    }

    @Transactional
    @Override
    public void deleteById(Integer markerId) {
        markerDao.delete(markerId);
    }

    @Override
    public void save(MarkGroup group, Integer[] userIds) {
        List<Marker> list = new ArrayList<Marker>();
        if (userIds != null && userIds.length > 0) {
            for (Integer userId : userIds) {
                Marker marker = this.findByExamAndSubjectAndNumberAndUserId(group.getExamId(), group.getSubjectCode(),
                        group.getNumber(), userId);
                if (marker == null) {
                    marker = new Marker();
                    marker.setExamId(group.getExamId());
                    marker.setSubjectCode(group.getSubjectCode());
                    marker.setGroupNumber(group.getNumber());
                    marker.setUserId(userId);
                    marker.setEnable(true);
                    list.add(marker);
                }
            }
        }
        this.batchSave(list);
    }

    @Override
    public List<Marker> findByExamAndSubjectAndUserIdAndEnable(Integer examId, String subjectCode, Integer userId,
            boolean enable) {
        return markerDao.findByExamIdAndSubjectCodeAndUserIdAndEnable(examId, subjectCode, userId, enable);
    }
}
