package cn.com.qmth.stmms.biz.user.model;

import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.UserSource;
import cn.com.qmth.stmms.common.utils.AccessControlUtils;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;

@Entity
@Table(name = "b_user")
public class User implements Serializable {

    private static final long serialVersionUID = -2094897723165518198L;

    public static final int ACCESS_TOKEN_EXPIRE_SECOND = 3600 * 24;

    /**
     * 主键
     */
    @Id
    @GeneratedValue
    private Integer id;

    /**
     * 登录名
     */
    @Column(name = "login_name", length = 64, nullable = false)
    private String loginName;

    /**
     * 名称
     */
    @Column(name = "name", length = 64, nullable = false)
    private String name;

    /**
     * 密码
     */
    @Column(name = "password", length = 64, nullable = false)
    private String password;

    /**
     * 角色
     */
    @Column(name = "role", length = 16, nullable = false)
    @Enumerated(EnumType.STRING)
    private Role role;

    /**
     * 来源
     */
    @Column(name = "source", length = 16, nullable = false)
    @Enumerated(EnumType.STRING)
    private UserSource source;

    /**
     * 状态
     */
    @Column(name = "enable", nullable = false)
    private boolean enable;

    /**
     * 所属学校ID
     */
    @Column(name = "school_id", nullable = false)
    private Integer schoolId;

    /**
     * 关联的外部用户标识
     */
    @Column(name = "related_account", length = 64, nullable = true)
    private String relatedAccount;

    /**
     * 最后一次登录时间
     */
    @Column(name = "last_login_time", nullable = true)
    private Date lastLoginTime;

    /**
     * 最后一次登录IP
     */
    @Column(name = "last_login_ip", length = 64, nullable = true)
    private String lastLoginIp;

    @Column(name = "created_time", nullable = false)
    private Date createdTime;

    @Column(name = "updated_time", nullable = false)
    private Date updatedTime;

    /**
     * 访问令牌
     */
    @Column(name = "access_token", length = 64, nullable = true)
    private String accessToken;

    /**
     * 访问令牌的刷新时间
     */
    @Column(name = "access_token_refresh_time", nullable = true)
    private Date accessTokenRefreshTime;

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getLoginName() {
        return loginName;
    }

    public void setLoginName(String loginName) {
        this.loginName = loginName;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public Role getRole() {
        return role;
    }

    public void setRole(Role role) {
        this.role = role;
    }

    public Integer getSchoolId() {
        return schoolId;
    }

    public void setSchoolId(Integer schoolId) {
        this.schoolId = schoolId;
    }

    public Date getLastLoginTime() {
        return lastLoginTime;
    }

    public void setLastLoginTime(Date lastLoginTime) {
        this.lastLoginTime = lastLoginTime;
    }

    public String getLastLoginIp() {
        return lastLoginIp;
    }

    public void setLastLoginIp(String lastLoginIp) {
        this.lastLoginIp = lastLoginIp;
    }

    public Date getCreatedTime() {
        return createdTime;
    }

    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    public Date getUpdatedTime() {
        return updatedTime;
    }

    public void setUpdatedTime(Date updatedTime) {
        this.updatedTime = updatedTime;
    }

    public UserSource getSource() {
        return source;
    }

    public void setSource(UserSource source) {
        this.source = source;
    }

    public String getRelatedAccount() {
        return relatedAccount;
    }

    public void setRelatedAccount(String relatedAccount) {
        this.relatedAccount = relatedAccount;
    }

    public boolean isEnable() {
        return enable;
    }

    public void setEnable(boolean enable) {
        this.enable = enable;
    }

    public String getAccessToken() {
        return accessToken;
    }

    public Date getAccessTokenRefreshTime() {
        return accessTokenRefreshTime;
    }

    public void refreshAccessToken() {
        this.accessToken = AccessControlUtils.randomString();
        this.accessTokenRefreshTime = new Date();
    }

    public String getAccount() {
        return source == UserSource.INTERNAL ? loginName : relatedAccount;
    }
}
