package cn.com.qmth.stmms.admin.exam;

import cn.com.qmth.stmms.admin.utils.SessionExamUtils;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.mark.model.PictureConfigItem;
import cn.com.qmth.stmms.common.controller.BaseController;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.ExamType;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.LinkedList;
import java.util.List;

public class BaseExamController extends BaseController {

    protected static Logger log = LoggerFactory.getLogger(BaseExamController.class);

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamService examService;

    protected Integer getSessionExamId(HttpServletRequest request) {
        return SessionExamUtils.getExamId(request);
    }

    /**
     * 添加Flash消息
     *
     * @param messages 消息
     */
    protected void addMessage(RedirectAttributes redirectAttributes, String... messages) {
        StringBuilder sb = new StringBuilder();
        for (String message : messages) {
            sb.append(message).append(messages.length > 1 ? "<br/>" : "");
        }
        redirectAttributes.addFlashAttribute("message", sb.toString());
    }

    protected List<Exam> getExamList(WebUser wu, ExamType type) {
        return examService.findBySchoolIdAndTypeAndStatus(wu.getUser().getSchoolId(), type);
    }

    protected List<Exam> getExamList(WebUser wu) {
        return examService.findBySchoolId(wu.getUser().getSchoolId());
    }

    protected List<ExamSubject> getExamSubject(Integer examId, WebUser wu) {
        List<ExamSubject> list = new LinkedList<ExamSubject>();
        if (wu.isSchoolAdmin() || wu.isSchoolViewer()) {
            list = subjectService.list(examId);
        } else if (wu.isSubjectHeader() || wu.isInspector()) {
            list.addAll(subjectService.list(examId, wu.getSubjectCodeSet()));
        }
        return list;
    }

    protected boolean subjectCheck(String subjectCode, WebUser wu) {
        if (wu.isSubjectHeader() || wu.isInspector()) {
            for (String code : wu.getSubjectCodeSet()) {
                if (code.equalsIgnoreCase(subjectCode)) {
                    return true;
                }
            }
            return false;
        } else {
            return true;
        }
    }

    protected String buildPictureConfig(String sheetConfig) {
        if (StringUtils.isNotBlank(sheetConfig)) {
            try {
                ObjectMapper mapper = new ObjectMapper();
                return mapper.writeValueAsString(PictureConfigItem.parse(sheetConfig));
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }
        }
        return "";
    }

}
