package cn.com.qmth.stmms.admin.report;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.admin.exam.BaseExamController;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectQuestion;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectQuestionService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;

/**
 * 客、主观题 成绩分析
 *
 * @author xuwenjin
 * @date 2019-04-18
 */
@Controller
@RequestMapping("/admin/exam/reportSubjectQuestion")
public class ReportSubjectQuestionController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(ReportSubjectQuestionController.class);

    @Autowired
    private ReportSubjectQuestionService reportSubjectQuestionService;

    @Autowired
    private ExamQuestionService questionService;

    @Logging(menu = "课程小题分析", type = LogType.QUERY)
    @RequestMapping
    public String list(Model model, HttpServletRequest request, ReportSubjectQuery query) {
        WebUser webuser = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        if (StringUtils.isNotBlank(query.getSubjectCode()) && query.getObjective() != null) {
            List<ReportSubjectQuestion> questions = reportSubjectQuestionService.findByQuery(query);
            questions = new ArrayList<>(questions);
            Collections.sort(questions, new Comparator<ReportSubjectQuestion>() {

                @Override
                public int compare(ReportSubjectQuestion o1, ReportSubjectQuestion o2) {
                    int i = o1.getMainNumber() - o2.getMainNumber();
                    if (i == 0) {
                        if (o1.getSubNumber().contains("-")) {
                            String[] o1s = o1.getSubNumber().split("-");
                            String[] o2s = o2.getSubNumber().split("-");
                            int j = Integer.parseUnsignedInt(o1s[0]) - Integer.parseUnsignedInt(o2s[0]);
                            if (j == 0) {
                                return Integer.parseUnsignedInt(o1s[1]) - Integer.parseUnsignedInt(o2s[1]);
                            } else {
                                return j;
                            }
                        } else {
                            return Integer.parseUnsignedInt(o1.getSubNumber())
                                    - Integer.parseUnsignedInt(o2.getSubNumber());
                        }
                    }
                    return i;
                }
            });
            model.addAttribute("list", questions);
            model.addAttribute("typeList", questionService.getPaperType(examId, query.getSubjectCode()));
        }
        model.addAttribute("subjectList", getExamSubject(examId, webuser));
        model.addAttribute("query", query);
        return "modules/report/reportSubjectQuestion";
    }

    @Logging(menu = "课程小题分析导出", type = LogType.EXPORT)
    @RequestMapping("/export")
    public String export(ReportSubjectQuery query, HttpServletRequest request, HttpServletResponse response,
            RedirectAttributes redirectAttributes) {
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        List<ReportSubjectQuestion> list = reportSubjectQuestionService.findByQuery(query);
        Collections.sort(list, new Comparator<ReportSubjectQuestion>() {

            @Override
            public int compare(ReportSubjectQuestion o1, ReportSubjectQuestion o2) {
                int i = o1.getMainNumber() - o2.getMainNumber();
                if (i == 0) {
                    if (o1.getSubNumber().contains("-")) {
                        String[] o1s = o1.getSubNumber().split("-");
                        String[] o2s = o2.getSubNumber().split("-");
                        int j = Integer.parseUnsignedInt(o1s[0]) - Integer.parseUnsignedInt(o2s[0]);
                        if (j == 0) {
                            return Integer.parseUnsignedInt(o1s[1]) - Integer.parseUnsignedInt(o2s[1]);
                        } else {
                            return j;
                        }
                    } else {
                        return Integer.parseUnsignedInt(o1.getSubNumber())
                                - Integer.parseUnsignedInt(o2.getSubNumber());
                    }
                }
                return i;
            }
        });
        for (ReportSubjectQuestion r : list) {
            r.setAvgScore(new BigDecimal(r.getAvgScore()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setStdev(new BigDecimal(r.getStdev()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setScoreRate(new BigDecimal(r.getScoreRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setFullScoreRate(new BigDecimal(r.getFullScoreRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
        }
        String fileName = query.getObjective() ? "客观题统计分析.xlsx" : "主观题统计分析.xlsx";
        try {
            new ExportExcel(query.getObjective() ? "客观题统计分析" : "主观题统计分析", ReportSubjectQuestion.class)
                    .setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导出成绩失败！" + e.getMessage());
            return "redirect:/admin/exam/reportSubjectQuestion";
        }
    }
}
