package cn.com.qmth.stmms.admin.report;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.admin.dto.ReportSubjectRangeDTO;
import cn.com.qmth.stmms.admin.exam.BaseExamController;
import cn.com.qmth.stmms.admin.thread.ScoreReportThread;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.report.model.ReportSubject;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;

/**
 * 课程分段统计
 *
 * @author xuwenjin
 * @date 2019-04-12 17:05
 */
@Controller
@RequestMapping("/admin/exam/reportSubjectRange")
public class ReportSubjectRangeController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(ReportSubjectRangeController.class);

    @Autowired
    private ReportSubjectService reportSubjectService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private LockService lockService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamService examService;

    @Autowired
    private ReportService reportService;

    @Logging(menu = "课程分段统计", type = LogType.QUERY)
    @RequestMapping
    public String list(Model model, HttpServletRequest request, ReportSubjectQuery query) {
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        if (StringUtils.isNotBlank(query.getSubjectCode()) && query.getRange() != null) {
            ReportSubject subject = reportSubjectService.findOne(examId, query.getSubjectCode());
            if (subject != null && subject.getScoreRange() != null && subject.getTotalScore() != null
                    && subject.getRealityCount() != null) {
                model.addAttribute(
                        "total",
                        getScoreRange(subject.getScoreRange(), subject.getTotalScore(), subject.getRealityCount(),
                                query.getRange()));
            }
            model.addAttribute("locked", lockService.isLocked(LockType.SCORE_CALCULATE, examId, query.getSubjectCode()));
        }
        model.addAttribute("subjectList", getExamSubject(examId, wu));
        model.addAttribute("query", query);
        return "modules/report/reportSubjectRange";
    }

    private JSONArray getScoreRange(String scoreRange, double totalScore, Integer totalCount, int range) {
        JSONArray result = new JSONArray();
        JSONObject jsonObject = JSONObject.fromObject(scoreRange);
        int rangeCount = 0;
        int sumCount = 0;
        int total = (int) Math.ceil(totalScore / range);
        for (int i = total; i >= 0; i--) {
            int start = i * range;
            int end = (i - 1) * range + 1;
            if (start > totalScore) {
                start = (int) Math.ceil(totalScore);
            }
            if (end < 0) {
                end = 0;
            }
            rangeCount = getSumCount(jsonObject, start, end);
            sumCount = sumCount + rangeCount;
            result.add(getRangeJson(totalCount, rangeCount, sumCount, i * range));
        }
        return result;
    }

    private int getSumCount(JSONObject jsonObject, int start, int end) {
        int sumCount = 0;
        int currentCount = 0;
        if (start < end) {
            for (int i = start; i <= end; i++) {
                currentCount = jsonObject.getInt(String.valueOf(i));
                sumCount = sumCount + currentCount;
            }
        } else if (start >= end) {
            for (int i = end; i <= start; i++) {
                currentCount = jsonObject.getInt(String.valueOf(i));
                sumCount = sumCount + currentCount;
            }
        } else {
            sumCount = jsonObject.getInt(String.valueOf(start));
        }
        return sumCount;
    }

    private JSONObject getRangeJson(Integer totalCount, int rangeCount, int sumCount, int score) {
        JSONObject value = new JSONObject();
        value.accumulate("score", score);
        value.accumulate("rangeCount", rangeCount);
        value.accumulate("rangeRate", rangeCount * 100.0 / totalCount);
        value.accumulate("sumCount", sumCount);
        value.accumulate("sumRate", sumCount * 100.0 / totalCount);
        return value;
    }

    @Logging(menu = "课程分段统计导出", type = LogType.EXPORT)
    @RequestMapping("/export")
    public String export(ReportSubjectQuery query, HttpServletRequest request, HttpServletResponse response,
            RedirectAttributes redirectAttributes) {
        List<ReportSubjectRangeDTO> list = new ArrayList<ReportSubjectRangeDTO>();
        int examId = getSessionExamId(request);
        if (StringUtils.isNotBlank(query.getSubjectCode()) && query.getRange() != null) {
            ReportSubject subject = reportSubjectService.findOne(examId, query.getSubjectCode());
            if (subject != null && subject.getScoreRange() != null && subject.getTotalScore() != null
                    && subject.getRealityCount() != null) {
                JSONArray array = getScoreRange(subject.getScoreRange(), subject.getTotalScore(),
                        subject.getRealityCount(), query.getRange());
                for (int i = 0; i < array.size(); i++) {
                    JSONObject jsonObject = array.getJSONObject(i);
                    String score = jsonObject.getInt("score") + "-";
                    Integer rangeCount = jsonObject.getInt("rangeCount");
                    Double rangeRate = jsonObject.getDouble("rangeRate");
                    list.add(new ReportSubjectRangeDTO(score, rangeCount, rangeRate));
                }
            }
        }
        String fileName = "课程分段统计.xlsx";
        try {
            new ExportExcel("课程分段统计", ReportSubjectRangeDTO.class).setDataList(list).write(response, fileName)
                    .dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导出成绩失败！" + e.getMessage());
            return "redirect:/admin/exam/reportSubjectRange";
        }
    }

    @Logging(menu = "科目成绩分析计算", type = LogType.UPDATE)
    @RequestMapping("/report")
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER })
    public String calculate(HttpServletRequest request, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode) {
        int examId = getSessionExamId(request);
        Set<String> subjectSet = new HashSet<String>();
        if (!lockService.isLocked(LockType.SCORE_CALCULATE, examId, subjectCode)) {
            subjectSet.add(subjectCode);
        }
        if (!subjectSet.isEmpty()) {
            ScoreReportThread thread = new ScoreReportThread(examId, subjectSet, lockService, studentService,
                    questionService, reportService, examService, subjectService, false);
            taskExecutor.submit(thread);
        }
        return "redirect:/admin/exam/reportSubjectRange?subjectCode=" + subjectCode + "&range=" + 10;
    }
}
