package cn.com.qmth.stmms.api.controller;

import cn.com.qmth.stmms.api.exception.ApiException;
import cn.com.qmth.stmms.common.controller.BaseController;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ExceptionHandler;

import javax.servlet.http.HttpServletResponse;

public class BaseApiController extends BaseController {

    protected static final Logger log = LoggerFactory.getLogger(BaseApiController.class);

    private static final String ERROR_MESSAGE_HEADER_KEY = "error-info";

    @ExceptionHandler
    public void exception(HttpServletResponse response, Exception ex) {
        log.error("api execute error", ex);
        response.addHeader(ERROR_MESSAGE_HEADER_KEY, StringUtils.trimToEmpty(ex.getMessage()));
        try {
            if (ex instanceof ApiException) {
                ApiException e = (ApiException) ex;
                response.sendError(e.getCode());
            } else {
                response.sendError(HttpStatus.INTERNAL_SERVER_ERROR.value());
            }
        } catch (Exception e) {
            log.error("api response senderror", e);
        }
    }

    protected JSONObject result(boolean success) {
        JSONObject obj = new JSONObject();
        obj.accumulate("success", success);
        return obj;
    }

    protected JSONObject result(boolean success, String url) {
        JSONObject obj = new JSONObject();
        obj.accumulate("success", success);
        obj.accumulate("url", url);
        return obj;
    }

    protected JSONObject result(String key, String value) {
        JSONObject obj = new JSONObject();
        obj.accumulate(key, value);
        return obj;
    }

    protected String validate(String name, String value, boolean required, int maxLength) {
        value = StringUtils.trimToNull(value);
        if (required && value == null) {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage(name + " is required");
        }
        if (value != null && maxLength > 0 && value.length() > maxLength) {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage(name + " length exceed " + maxLength);
        }
        return value;
    }

    protected String validate(String value, boolean notBlank, int maxLength) {
        if (value == null) {
            return null;
        }
        value = value.trim();
        if (notBlank && value.length() == 0) {
            return null;
        }
        if (maxLength > 0 && value.length() > maxLength) {
            return null;
        }
        return value;
    }
}
