# ************************************************************
# Database: stmms_fenti
# Generation Time: 2017-10-31 06:43:10 +0000
# ************************************************************


# Dump of table b_school
# ------------------------------------------------------------

DROP TABLE IF EXISTS `b_school`;

CREATE TABLE `b_school`
(
    `id`            int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `name`          varchar(64) NOT NULL COMMENT '名称',
    `province`      varchar(16) NOT NULL COMMENT '省份',
    `city`          varchar(16) NOT NULL COMMENT '城市',
    `code`          varchar(64)  DEFAULT NULL COMMENT '代码',
    `address`       varchar(128) DEFAULT NULL COMMENT '地址',
    `phone`         varchar(32)  DEFAULT NULL COMMENT '电话',
    `logo_url`      varchar(64)  DEFAULT NULL COMMENT '图片地址',
    `description`   varchar(128) DEFAULT NULL COMMENT '描述',
    `enable`        tinyint(1)  NOT NULL COMMENT '是否禁用',
    `access_key`    varchar(64)  DEFAULT NULL COMMENT 'AccessKey',
    `access_secret` varchar(64)  DEFAULT NULL COMMENT 'AccessSecret',
    `create_time`   datetime    NOT NULL COMMENT '创建时间',
    `update_time`   datetime     DEFAULT NULL COMMENT '修改时间',
    PRIMARY KEY (`id`),
    UNIQUE KEY `index1` (`access_key`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='学校表';


# Dump of table b_user
# ------------------------------------------------------------

DROP TABLE IF EXISTS `b_user`;

CREATE TABLE `b_user`
(
    `id`                        int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `login_name`                varchar(64) NOT NULL COMMENT '登录名',
    `name`                      varchar(64) NOT NULL COMMENT '名称',
    `password`                  varchar(64) NOT NULL COMMENT '密码',
    `role`                      varchar(16) NOT NULL COMMENT '角色',
    `source`                    varchar(16) NOT NULL COMMENT '来源',
    `enable`                    tinyint(1)  NOT NULL COMMENT '是否启用',
    `school_id`                 int(11)     NOT NULL COMMENT '所属学校ID',
    `related_account`           varchar(64) DEFAULT NULL COMMENT '关联外部用户',
    `last_login_time`           datetime    DEFAULT NULL COMMENT '最后一次登录时间',
    `last_login_ip`             varchar(64) DEFAULT NULL COMMENT '最后一次登录IP',
    `created_time`              datetime    NOT NULL COMMENT '创建时间',
    `updated_time`              datetime    DEFAULT NULL COMMENT '修改时间',
    `access_token`              varchar(64) DEFAULT NULL COMMENT '访问令牌',
    `access_token_refresh_time` datetime    DEFAULT NULL COMMENT '访问令牌刷新时间',
    PRIMARY KEY (`id`),
    UNIQUE KEY `index1` (`login_name`),
    UNIQUE KEY `index2` (`school_id`, `related_account`),
    KEY `index3` (`school_id`, `role`, `source`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='用户表';

LOCK TABLES `b_user` WRITE;

INSERT INTO `b_user` (`id`, `login_name`, `name`, `password`, `role`, `source`, `enable`, `school_id`,
                      `created_time`, `updated_time`)
VALUES (1, 'admin', '超级管理员', 'd0e915689fc1f1e19e1a036c22bdfeab', 'SYS_ADMIN', 'INTERNAL', 1, 0,
        '2020-08-01 12:00:00', '2020-08-01 12:00:00');

UNLOCK TABLES;


# Dump of table eb_subject_user
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_subject_user`;

CREATE TABLE `eb_subject_user`
(
    `id`           int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `user_id`      int(11)     NOT NULL COMMENT '用户ID',
    `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
    PRIMARY KEY (`id`),
    UNIQUE KEY `index1` (`user_id`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='用户科目关联表';

# Dump of table eb_check_student
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_check_student`;
CREATE TABLE `eb_check_student`
(
    `student_id`   int(11)     NOT NULL COMMENT '学生ID',
    `exam_id`      int(11)     NOT NULL COMMENT '考试ID',
    `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
    `type`         varchar(16) NOT NULL COMMENT '类型',
    `is_checked`   tinyint(1)  NOT NULL COMMENT '是否已处理',
    `update_time`  datetime DEFAULT NULL COMMENT '处理时间',
    PRIMARY KEY (`student_id`),
    KEY `index1` (`exam_id`, `subject_code`, `type`, `is_checked`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='审核学生表';

# Dump of table eb_data_sync
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_data_sync`;
CREATE TABLE `eb_data_sync`
(
    `examId`        int(11)      NOT NULL COMMENT '考试ID',
    `school_id`     int(11)      DEFAULT NULL COMMENT '学校ID',
    `cloud_exam_id` bigint(20)   NOT NULL COMMENT '云平台考试ID',
    `finished`      tinyint(1)   NOT NULL COMMENT '是否完成',
	`source`		varchar(16)  NOT NULL COMMENT '数据来源',
    `next_id`       bigint(20)   DEFAULT NULL COMMENT '下一位考生ID',
    `root_org_id`   varchar(32)  DEFAULT NULL COMMENT '机构ID',
    `subject_code`  varchar(32)  DEFAULT NULL COMMENT '科目代码',
    `app_id`        varchar(128) DEFAULT NULL COMMENT '项目ID',
	`access_key` 	varchar(64) DEFAULT NULL COMMENT '密匙',
  	`access_secret` varchar(64) DEFAULT NULL COMMENT '密钥',
    `student_url`   varchar(128) NOT NULL COMMENT '考生API URL',
    `subject_url`   varchar(128) NOT NULL COMMENT '科目API URL',
    `subject_paper_url` varchar(255) NOT NULL COMMENT '试卷详情API URL',
    `create_time`   datetime    DEFAULT NULL COMMENT '创建时间',
    `update_time`   datetime    DEFAULT NULL COMMENT '更新时间',
    PRIMARY KEY (`examId`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='数据同步表';

# Dump of table eb_exam
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam`;

CREATE TABLE `eb_exam`
(
    `id`                int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `name`              varchar(64) NOT NULL COMMENT '考试名称',
    `school_id`         int(11)     NOT NULL COMMENT '学校ID',
    `code`              varchar(64)  DEFAULT NULL COMMENT '考试代码',
    `exam_time`         datetime    NOT NULL COMMENT '考试时间',
    `type`              varchar(16) NOT NULL COMMENT '类型',
    `status`            varchar(16) NOT NULL COMMENT '状态',
    `card_type`         varchar(16)  DEFAULT NULL COMMENT '题卡类型',
    `force_special_tag` tinyint(1)  NOT NULL COMMENT '强制标记',
    `forbidden_info`    tinyint(1)  NOT NULL COMMENT '是否屏蔽个人信息',
    `forbidden_score`   tinyint(1)  NOT NULL COMMENT '是否屏蔽查询个人信息',
    `objective_status`  varchar(16) NOT NULL COMMENT '客观题统分状态',
    `sas_config`        text         DEFAULT NULL COMMENT '统计配置',
    `sheet_config`      text         DEFAULT NULL COMMENT '原图遮盖配置',
    `slice_config`      text         DEFAULT NULL COMMENT '裁切图坐标',
    `description`       varchar(128) DEFAULT NULL COMMENT '描述',
    `creator_id`        int(11)      DEFAULT NULL COMMENT '创建人',
    `create_time`       datetime     NOT NULL COMMENT '创建时间',
    `update_time`       datetime     DEFAULT NULL COMMENT '修改时间',
    `mark_start_time`   datetime     DEFAULT NULL COMMENT '考试开始时间',
    `mark_end_time`     datetime     DEFAULT NULL COMMENT '考试结束时间',
	`sheet_view`		tinyint(1)   NOT NULL COMMENT '原卷显示功能',
    `mark_mode`       	varchar(16)  DEFAULT NULL COMMENT '强制评卷模式',
    PRIMARY KEY (`id`),
    UNIQUE KEY `index1` (`school_id`, `code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='考试表';



# Dump of table eb_exam_package
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam_package`;

CREATE TABLE `eb_exam_package`
(
    `exam_id`   int(11)     NOT NULL COMMENT '考试ID',
    `code`      varchar(64) NOT NULL COMMENT '试卷袋编号',
    `pic_count` int(11)     NOT NULL COMMENT '上传图片数量',
    PRIMARY KEY (`exam_id`, `code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='卷袋信息表';



# Dump of table eb_exam_question
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam_question`;

CREATE TABLE `eb_exam_question`
(
    `id`               int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`          int(11)     NOT NULL COMMENT '考试ID',
    `subject_code`     varchar(32) NOT NULL DEFAULT '' COMMENT '科目代码',
    `paper_type`       varchar(8)  NOT NULL COMMENT '试卷类型',
    `is_objective`     tinyint(1)  NOT NULL COMMENT '是否客观题',
    `group_number`     int(11)     DEFAULT NULL COMMENT '分组序号',
    `main_number`      int(11)     NOT NULL COMMENT '大题号',
    `sub_number`       varchar(32) NOT NULL COMMENT '小题号',
    `main_title`       varchar(128) NOT NULL COMMENT '大题名称',
    `answer`           varchar(16)          DEFAULT NULL COMMENT '正确答案',
    `total_score`      double      NOT NULL COMMENT '满分',
    `interval_score`   double      NOT NULL COMMENT '评卷间隔分',
    `objective_policy` varchar(16)          DEFAULT NULL COMMENT '客观题判分策略',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`, `is_objective`, `main_number`, `sub_number`, `paper_type`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='小题信息表';



# Dump of table eb_exam_student
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam_student`;

CREATE TABLE `eb_exam_student`
(
    `id`                    int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`               int(11)     NOT NULL COMMENT '考试ID',
    `school_id`             int(11)     NOT NULL COMMENT '学校ID',
    `subject_code`          varchar(32) NOT NULL COMMENT '科目代码',
    `subject_name`          varchar(32) NOT NULL COMMENT '科目名称',
    `paper_type`            varchar(8)  NOT NULL COMMENT '试卷类型',
    `exam_number`           varchar(64) NOT NULL COMMENT '准考证号',
    `secret_number`         varchar(64) NOT NULL COMMENT '密号',
    `student_code`          varchar(64) NOT NULL COMMENT '学号',
    `name`                  varchar(64) NOT NULL COMMENT '姓名',
    `package_code`          varchar(64)  DEFAULT NULL COMMENT '试卷袋编号',
    `campus_name`           varchar(64)  DEFAULT NULL COMMENT '学习中心名称',
    `exam_site`             varchar(32)  DEFAULT NULL COMMENT '考点',
    `exam_room`             varchar(32)  DEFAULT NULL COMMENT '考场',
    `remark`                varchar(128) DEFAULT NULL COMMENT '备注',
    `batch_code`            varchar(32)  DEFAULT NULL COMMENT '扫描批次号',
    `sheet_count`           int(11)     NOT NULL COMMENT '原图数量',
    `slice_count`           int(11)     NOT NULL COMMENT '小图数量',
    `answers`               text COMMENT '客观题识别结果',
    `is_upload`             tinyint(1)  NOT NULL COMMENT '是否已上传',
    `is_absent`             tinyint(1)  NOT NULL COMMENT '是否缺考',
    `is_manual_absent`      tinyint(1)  NOT NULL COMMENT '是否人工指定缺考',
    `is_breach`             tinyint(1)  NOT NULL COMMENT '是否违纪',
    `is_exception`          tinyint(1)  NOT NULL COMMENT '是否数据异常',
    `upload_time`           datetime     DEFAULT NULL COMMENT '上传时间',
    `inspect_time`          datetime     DEFAULT NULL COMMENT '复核时间',
    `inspector_id`          int(11)      DEFAULT NULL COMMENT '复核人ID',
    `objective_score`       double       DEFAULT NULL COMMENT '客观总分',
    `objective_score_list`  text COMMENT '客观得分明细',
    `subjective_status`     varchar(16) NOT NULL COMMENT '主观题状态',
    `subjective_score`      double       DEFAULT NULL COMMENT '主观总分',
    `subjective_score_list` text COMMENT '主观得分明细',
    `subject_level`         varchar(64)  DEFAULT NULL COMMENT '层次',
    `subject_category`      varchar(64)  DEFAULT NULL COMMENT '专业类型',
    `college`               varchar(64) NOT NULL COMMENT '学院',
    `class_name`            varchar(64) NOT NULL COMMENT '班级',
    `teacher`               varchar(64) NOT NULL COMMENT '任课老师',
    PRIMARY KEY (`id`),
    UNIQUE KEY `index1` (`exam_id`, `exam_number`),
    UNIQUE KEY `index2` (`exam_id`, `secret_number`),
    KEY `index3` (`exam_id`, `subject_code`, `upload_time`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='考试考生库';



# Dump of table eb_exam_subject
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_exam_subject`;

CREATE TABLE `eb_exam_subject`
(
    `exam_id`          int(11)     NOT NULL COMMENT '考试ID',
    `code`             varchar(32) NOT NULL COMMENT '科目代码',
    `name`             varchar(32) NOT NULL COMMENT '科目名称',
    `level`            varchar(64)  DEFAULT NULL COMMENT '层次',
    `category`         varchar(64)  DEFAULT NULL COMMENT '专业类型',
    `objective_score`  double      NOT NULL COMMENT '客观题满分',
    `subjective_score` double      NOT NULL COMMENT '主观题满分',
    `total_score`      double      NOT NULL COMMENT '全卷满分',
    `upload_count`     int(11)     NOT NULL COMMENT '已上传人数',
    `remark`           varchar(128) DEFAULT NULL COMMENT '备注',
    `slice_config`     text         DEFAULT NULL COMMENT '裁切图配置',
    `sheet_config`     text         DEFAULT NULL COMMENT '原图遮盖配置',
    `sas_config`       text         DEFAULT NULL COMMENT '统计配置',
    `card_type`        varchar(16)  DEFAULT NULL COMMENT '题卡类型',
    `paper_file_type`  varchar(16)  DEFAULT NULL COMMENT '试卷文件类型',
    `answer_file_type` varchar(16)  DEFAULT NULL COMMENT '标答文件类型',
    PRIMARY KEY (`exam_id`, `code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='考试科目表';

# Dump of table eb_marker
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_marker`;

CREATE TABLE `eb_marker`
(
    `id`           int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`      int(11)     NOT NULL COMMENT '考试ID',
    `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
    `group_number` int(11)     NOT NULL COMMENT '分组编号',
    `user_id`      int(11)     NOT NULL COMMENT '用户ID',
    `enable`       tinyint(1)  NOT NULL COMMENT '是否启用',
    `mode`         varchar(16) DEFAULT NULL COMMENT '强制评卷模式',
    `top_count`    int(11)     DEFAULT NULL COMMENT '评卷数上限',
    `finish_count` int(11)     DEFAULT NULL COMMENT '完成数量',
    `valid_count`  int(11)     DEFAULT NULL COMMENT '有效数量',
    `avg_score`    double      DEFAULT NULL COMMENT '平均分',
    `avg_speed`    double      DEFAULT NULL COMMENT '平均时长',
    `stdev_score`  double      DEFAULT NULL COMMENT '标准差',
    `mark_setting` text        DEFAULT NULL COMMENT '个性化评卷参数设置',
    PRIMARY KEY (`id`),
    KEY `index1` (`user_id`),
    KEY `index2` (`exam_id`, `subject_code`, `group_number`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='评卷员表';


# Dump of table eb_marker_class
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_marker_class`;

CREATE TABLE `eb_marker_class`
(
    `id`         int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `user_id`    int(11) NOT NULL COMMENT '用户ID',
    `exam_id`    int(11) NOT NULL COMMENT '考试ID',
    `class_name` varchar(64) DEFAULT NULL COMMENT '班级名称',
    PRIMARY KEY (`id`),
    UNIQUE KEY `index1` (`user_id`, `exam_id`, `class_name`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='评卷员班级表';


# Dump of table eb_mark_group
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_mark_group`;

CREATE TABLE `eb_mark_group`
(
    `exam_id`             int(11)     NOT NULL COMMENT '考试ID',
    `subject_code`        varchar(32) NOT NULL COMMENT '科目代码',
    `number`              int(11)     NOT NULL COMMENT '序号',
    `pic_list`            varchar(255) DEFAULT NULL COMMENT '小图配置',
    `total_score`         double      NOT NULL COMMENT '满分',
    `trial_count`         int(11)      DEFAULT NULL COMMENT '试评数量',
    `double_rate`         double       DEFAULT NULL COMMENT '双评比例',
    `arbitrate_threshold` double       DEFAULT NULL COMMENT '仲裁阈值',
    `score_policy`        varchar(16)  DEFAULT NULL COMMENT '合分策略',
    `third_policy`        varchar(32)  DEFAULT NULL COMMENT '三评规则',
    `mark_mode`           varchar(16)  DEFAULT NULL COMMENT '强制评卷模式',
    `sheet_view`          tinyint(1)  NOT NULL COMMENT '是否允许查看原卷',
    `enable_all_zero`     tinyint(1)  NOT NULL COMMENT '是否启用全零分',
    `status`              varchar(16) NOT NULL COMMENT '评卷状态',
    `build_time`          datetime     DEFAULT NULL COMMENT '最后生成任务时间',
    `library_count`       int(11)     NOT NULL COMMENT '任务总量',
    `marked_count`        int(11)     NOT NULL COMMENT '已评数量',
    `left_count`          int(11)     NOT NULL COMMENT '剩余数量',
    PRIMARY KEY (`exam_id`, `subject_code`, `number`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='评卷分组表';


# Dump of table eb_mark_group_student
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_mark_group_student`;

CREATE TABLE `eb_mark_group_student`
(
    `student_id`   int(11)     NOT NULL COMMENT '考生ID',
    `group_number` int(11)     NOT NULL COMMENT '分组ID',
    `exam_id`      int(11)     NOT NULL COMMENT '考试ID',
    `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
    `status`       varchar(16) NOT NULL COMMENT '状态',
    PRIMARY KEY (`student_id`, `group_number`),
    KEY `index1` (`exam_id`, `subject_code`, `group_number`, `status`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='考生分组状态表';


# Dump of table eb_subjective_score
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_subjective_score`;

CREATE TABLE `eb_subjective_score`
(
    `student_id`   int(11)     NOT NULL COMMENT '考生ID',
    `main_number`  int(11)     NOT NULL COMMENT '大题号',
    `sub_number`   varchar(32) NOT NULL COMMENT '小题号',
    `exam_id`      int(11)     NOT NULL COMMENT '考试ID',
    `subject_code` varchar(32) NOT NULL COMMENT '科目代码',
    `group_number` int(11)     NOT NULL COMMENT '分组序号',
    `group_score`  double      NOT NULL COMMENT '分组得分',
    `main_score`   double      NOT NULL COMMENT '大题得分',
    `score`        double      NOT NULL COMMENT '小题得分',
    PRIMARY KEY (`student_id`, `main_number`, `sub_number`),
    KEY `index1` (`exam_id`, `subject_code`, `main_number`, `main_score`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='主观题得分明细表';


# Dump of table eb_operation_log
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_operation_log`;

CREATE TABLE `eb_operation_log`
(
    `id`          int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`     int(11)      DEFAULT NULL COMMENT '考试ID',
    `school_id`   int(11)     NOT NULL COMMENT '学校ID',
    `type`        varchar(16) NOT NULL COMMENT '类型',
    `menu`        varchar(128) DEFAULT NULL COMMENT '菜单',
    `description` varchar(128) DEFAULT NULL COMMENT '详情',
    `is_marker`   tinyint(1)  NOT NULL COMMENT '是否评卷员',
    `operator_id` int(11)     NOT NULL COMMENT '操作人ID',
    `login_name`  varchar(64) NOT NULL COMMENT '登录名',
    `ip_address`  varchar(128) DEFAULT NULL COMMENT '访问地址',
    `create_time` datetime    NOT NULL COMMENT '创建时间',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `school_id`, `type`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='考试操作记录表';


# Dump of table m_arbitrate_history
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_arbitrate_history`;

CREATE TABLE `m_arbitrate_history`
(
    `id`            int(11)     NOT NULL AUTO_INCREMENT COMMENT '自增主键',
    `exam_id`       int(11)     NOT NULL COMMENT '考试ID',
    `subject_code`  varchar(32) NOT NULL COMMENT '科目代码',
    `group_number`  int(11)     NOT NULL COMMENT '大题号',
    `exam_number`   varchar(64) NOT NULL COMMENT '准考证号',
    `secret_number` varchar(64) NOT NULL COMMENT '考生密号',
    `student_id`    int(11)     NOT NULL COMMENT '考生ID',
    `status`        varchar(16) NOT NULL COMMENT '状态',
    `user_id`       int(11)      DEFAULT NULL COMMENT '处理人ID',
    `total_score`   double       DEFAULT NULL COMMENT '总分',
    `score_list`    varchar(255) DEFAULT NULL COMMENT '给分明细',
    `create_time`   datetime    NOT NULL COMMENT '创建时间',
    `update_time`   datetime     DEFAULT NULL COMMENT '处理时间',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`, `group_number`, `status`),
    KEY `index2` (`user_id`, `status`, `update_time`),
    KEY `index3` (`student_id`, `status`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='仲裁记录表';


# Dump of table m_library
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_library`;

CREATE TABLE `m_library`
(
    `id`                int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`           int(11)     NOT NULL COMMENT '考试ID',
    `subject_code`      varchar(32) NOT NULL COMMENT '科目代码',
    `group_number`      int(11)     NOT NULL COMMENT '大题号',
    `student_id`        int(11)     NOT NULL COMMENT '考生ID',
    `exam_number`       varchar(64) NOT NULL COMMENT '准考证号',
    `secret_number`     varchar(64) NOT NULL COMMENT '考生密号',
    `task_number`       int(11)     NOT NULL COMMENT '多评任务序号',
    `status`            varchar(16) NOT NULL COMMENT '状态',
    `marker_id`         int(11)      DEFAULT NULL COMMENT '评卷员ID',
    `marker_time`       datetime     DEFAULT NULL COMMENT '评卷时间',
    `marker_score`      double       DEFAULT NULL COMMENT '评卷总分',
    `marker_score_list` varchar(255) DEFAULT NULL COMMENT '评卷给分明细',
    `marker_spent`      int(11)      DEFAULT NULL COMMENT '最新评卷时间',
    `header_id`         int(11)      DEFAULT NULL COMMENT '科组长ID',
    `header_time`       datetime     DEFAULT NULL COMMENT '科组长评卷时间',
    `header_score`      double       DEFAULT NULL COMMENT '科组长总分',
    `header_score_list` varchar(255) DEFAULT NULL COMMENT '科组长给分明细',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`, `group_number`, `status`),
    UNIQUE KEY `index2` (`student_id`, `group_number`, `task_number`),
    KEY `index3` (`marker_id`, `status`, `marker_time`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='评卷任务表';


# Dump of table m_special_tag
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_special_tag`;

CREATE TABLE `m_special_tag`
(
    `id`           int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `library_id`   int(11)     NOT NULL COMMENT '评卷任务ID',
    `tag_name`     varchar(64) NOT NULL COMMENT '标记内容',
    `position_x`   double      NOT NULL COMMENT 'X轴位置',
    `position_y`   double      NOT NULL COMMENT 'Y轴位置',
    `offset_index` int(11)     NOT NULL COMMENT '裁切图序号',
    `offset_x`     int(11)     NOT NULL COMMENT '裁切图X轴坐标',
    `offset_y`     int(11)     NOT NULL COMMENT '裁切图Y轴坐标',
    PRIMARY KEY (`id`),
    KEY `index1` (`library_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='特殊标记表';


# Dump of table m_track
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_track`;

CREATE TABLE `m_track`
(
    `library_id`      int(11)      NOT NULL COMMENT '评卷任务ID',
    `question_number` varchar(128) NOT NULL COMMENT '完整题号',
    `number`          int(11)      NOT NULL COMMENT '序号',
    `student_id`      int(11)      NOT NULL COMMENT '考生ID',
    `exam_id`         int(11)      NOT NULL COMMENT '考试ID',
    `subject_code`    varchar(32)  NOT NULL COMMENT '科目代码',
    `group_number`    int(11)      NOT NULL COMMENT '大题题号',
    `marker_id`       int(11)      NOT NULL COMMENT '评卷员ID',
    `score`           double       NOT NULL COMMENT '给分',
    `position_x`      double       NOT NULL COMMENT 'X轴位置',
    `position_y`      double       NOT NULL COMMENT 'Y轴位置',
    `offset_index`    int(11)      NOT NULL COMMENT '裁切图序号',
    `offset_x`        int(11)      NOT NULL COMMENT '裁切图X轴坐标',
    `offset_y`        int(11)      NOT NULL COMMENT '裁切图Y轴坐标',
    PRIMARY KEY (`library_id`, `question_number`, `number`),
    KEY `index1` (`student_id`, `group_number`),
    KEY `index2` (`marker_id`),
    KEY `index3` (`exam_id`, `subject_code`, `group_number`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='轨迹给分表';


# Dump of table m_problem_type
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_problem_type`;
CREATE TABLE `m_problem_type`
(
    `id`        int(11)    NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`   int(11)    NOT NULL COMMENT '考试ID',
    `name`      varchar(64) DEFAULT NULL COMMENT '名称',
    `is_custom` tinyint(1) NOT NULL COMMENT '是否自定义',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='问题类型表';


# Dump of table m_problem_history
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_problem_history`;
CREATE TABLE `m_problem_history`
(
    `id`            int(11)     NOT NULL AUTO_INCREMENT COMMENT '自增主键',
    `exam_id`       int(11)     NOT NULL COMMENT '考试ID',
    `subject_code`  varchar(32) NOT NULL COMMENT '科目代码',
    `group_number`  int(11)     NOT NULL COMMENT '大题号',
    `exam_number`   varchar(64) NOT NULL COMMENT '准考证号',
    `secret_number` varchar(64) NOT NULL COMMENT '考生密号',
    `problem_id`    int(11)     NOT NULL COMMENT '问题ID',
    `student_id`    int(11)     NOT NULL COMMENT '考生ID',
    `library_id`    int(11)     NOT NULL COMMENT '评卷任务ID',
    `status`        varchar(16)  DEFAULT NULL COMMENT '状态',
    `user_id`       int(11)      DEFAULT NULL COMMENT '处理人ID',
    `total_score`   double       DEFAULT NULL COMMENT '总分',
    `score_list`    varchar(255) DEFAULT NULL COMMENT '给分明细',
    `create_time`   datetime    NOT NULL COMMENT '创建时间',
    `update_time`   datetime     DEFAULT NULL COMMENT '处理时间',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`, `group_number`, `status`),
    KEY `index2` (`user_id`, `status`, `update_time`),
    KEY `index3` (`student_id`, `status`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='问题卷历史表';


# Dump of table m_trial_history
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_trial_history`;
CREATE TABLE `m_trial_history`
(
    `library_id`        int(11)      NOT NULL COMMENT '评卷任务ID',
    `marker_id`         int(11)      NOT NULL COMMENT '评卷员ID',
    `exam_id`           int(11)      NOT NULL COMMENT '考试ID',
    `subject_code`      varchar(32)  NOT NULL COMMENT '科目代码',
    `group_number`      int(11)      NOT NULL COMMENT '分组序号',
    `student_id`        int(11)      NOT NULL COMMENT '考生ID',
    `secret_number`     varchar(64)  NOT NULL COMMENT '考生密号',
    `marker_time`       datetime     NOT NULL COMMENT '评卷时间',
    `marker_score`      double       NOT NULL COMMENT '评卷总分',
    `marker_score_list` varchar(255) NOT NULL COMMENT '评卷总分明细',
    PRIMARY KEY (`library_id`, `marker_id`),
    KEY `index1` (`exam_id`, `subject_code`, `group_number`),
    KEY `index2` (`student_id`),
    KEY `index3` (`marker_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='试评记录表';


# Dump of table m_trial_library
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_trial_library`;
CREATE TABLE `m_trial_library`
(
    `id`            int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`       int(11)     NOT NULL COMMENT '考试ID',
    `subject_code`  varchar(32) NOT NULL COMMENT '科目代码',
    `group_number`  int(11)     NOT NULL COMMENT '分组序号',
    `student_id`    int(11)     NOT NULL COMMENT '考生ID',
    `exam_number`   varchar(64) NOT NULL COMMENT '准考证号',
    `secret_number` varchar(64) NOT NULL COMMENT '考生密号',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`, `group_number`),
    UNIQUE KEY `index2` (`student_id`, `group_number`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='试评任务表';


# Dump of table m_trial_tag
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_trial_tag`;
CREATE TABLE `m_trial_tag`
(
    `id`           int(11)     NOT NULL AUTO_INCREMENT COMMENT '主键',
    `library_id`   int(11)     NOT NULL COMMENT '评卷任务ID',
    `marker_id`    int(11)     NOT NULL COMMENT '评卷员ID',
    `content`      varchar(64) NOT NULL COMMENT '标记内容',
    `position_x`   double      NOT NULL COMMENT 'X轴位置',
    `position_y`   double      NOT NULL COMMENT 'Y轴位置',
    `offset_index` int(11)     NOT NULL COMMENT '裁切图序号',
    `offset_x`     int(11)     NOT NULL COMMENT '裁切图X轴坐标',
    `offset_y`     int(11)     NOT NULL COMMENT '裁切图Y轴坐标',
    PRIMARY KEY (`id`),
    KEY `index1` (`library_id`, `marker_id`),
    KEY `index2` (`marker_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='试评特殊标记表';


# Dump of table m_trial_track
# ------------------------------------------------------------

DROP TABLE IF EXISTS `m_trial_track`;
CREATE TABLE `m_trial_track`
(
    `library_id`      int(11)     NOT NULL COMMENT '评卷任务ID',
    `marker_id`       int(11)     NOT NULL COMMENT '评卷员ID',
    `question_number` varchar(64) NOT NULL COMMENT '完整题号',
    `number`          int(11)     NOT NULL COMMENT '序号',
    `exam_id`         int(11)     NOT NULL COMMENT '考试ID',
    `subject_code`    varchar(32) NOT NULL COMMENT '科目代码',
    `group_number`    int(11)     NOT NULL COMMENT '大题题号',
    `student_id`      int(11)     NOT NULL COMMENT '考生ID',
    `score`           double      NOT NULL COMMENT '给分',
    `position_x`      double      NOT NULL COMMENT 'X轴位置',
    `position_y`      double      NOT NULL COMMENT 'Y轴位置',
    `offset_index`    int(11)     NOT NULL COMMENT '裁切图序号',
    `offset_x`        int(11)     NOT NULL COMMENT '裁切图X轴坐标',
    `offset_y`        int(11)     NOT NULL COMMENT '裁切图Y轴坐标',
    PRIMARY KEY (`library_id`, `marker_id`, `number`, `question_number`),
    KEY `index1` (`exam_id`, `subject_code`, `group_number`),
    KEY `index2` (`student_id`),
    KEY `index3` (`marker_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='试评轨迹表';



# Dump of table s_basic_group
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_class_group`;
CREATE TABLE `s_basic_class_group`
(
    `id`                   int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`              int(11)      DEFAULT NULL COMMENT '考试ID',
    `subject_code`         varchar(32)  DEFAULT NULL COMMENT '科目代码',
    `subject_name`         varchar(32)  DEFAULT NULL COMMENT '科目名称',
    `class_name`           varchar(64)  DEFAULT NULL COMMENT '班级名称',
    `discrimination`       double       DEFAULT NULL COMMENT '区分度',
    `objective`            tinyint(1)   DEFAULT NULL COMMENT '是否客观题',
    `group_name`           varchar(128) DEFAULT NULL COMMENT '大题名称',
    `group_number`         int(11)      DEFAULT NULL COMMENT '大题号',
    `question_count`       int(11)      DEFAULT NULL COMMENT '题目数量',
    `total_score`          double       DEFAULT NULL COMMENT '总分',
    `difficulity_level`    text         DEFAULT NULL COMMENT '难度分布',
    `discrimination_level` text         DEFAULT NULL COMMENT '区分度分布',
    `coefficient`          double       DEFAULT NULL COMMENT '差异系数',
    `difficulty`           double       DEFAULT NULL COMMENT '难度',
    `paper_type`           varchar(8)   DEFAULT NULL COMMENT '试卷类型',
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='班级大题统计表';


# Dump of table s_basic_group
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_group`;
CREATE TABLE `s_basic_group`
(
    `id`                   int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`              int(11)      DEFAULT NULL COMMENT '考试ID',
    `subject_code`         varchar(32)  DEFAULT NULL COMMENT '科目代码',
    `subject_name`         varchar(32)  DEFAULT NULL COMMENT '科目名称',
    `objective`            tinyint(1)   DEFAULT NULL COMMENT '是否客观题',
    `group_name`           varchar(128) DEFAULT NULL COMMENT '大题名称',
    `group_number`         int(11)      DEFAULT NULL COMMENT '大题号',
    `max_score`            double       DEFAULT NULL COMMENT '最高分',
    `min_score`            double       DEFAULT NULL COMMENT '最低分',
    `avg_score`            double       DEFAULT NULL COMMENT '平均分',
    `total_score`          double       DEFAULT NULL COMMENT '满分',
    `stdev`                double       DEFAULT NULL COMMENT '标准差',
    `coefficient`          double       DEFAULT NULL COMMENT '差异系数',
    `score_rate`           double       DEFAULT NULL COMMENT '得分率',
    `full_count`           int(11)      DEFAULT NULL COMMENT '满分人数',
    `zero_count`           int(11)      DEFAULT NULL COMMENT '零分人数',
    `difficulty`           double       DEFAULT NULL COMMENT '难度',
    `discrimination`       double       DEFAULT NULL COMMENT '区分度',
    `question_count`       int(11)      DEFAULT NULL COMMENT '题目数量',
    `reality_count`        int(11)      DEFAULT NULL COMMENT '有效数量',
    `difficulity_level`    text         DEFAULT NULL COMMENT '难度分布',
    `discrimination_level` text         DEFAULT NULL COMMENT '区分度分布',
    `paper_type`           varchar(8)   DEFAULT NULL COMMENT '试卷类型',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='大题统计表';


# Dump of table s_basic_question
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_question`;
CREATE TABLE `s_basic_question`
(
    `id`              int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`         int(11)     DEFAULT NULL COMMENT '考试ID',
    `subject_code`    varchar(32) DEFAULT NULL COMMENT '科目代码',
    `subject_name`    varchar(32) DEFAULT NULL COMMENT '科目名称',
    `objective`       tinyint(1)  DEFAULT NULL COMMENT '是否客观题',
    `question_name`   varchar(64) DEFAULT NULL COMMENT '题目名称',
    `paper_type`      varchar(8)  DEFAULT NULL COMMENT '试卷类型',
    `main_number`     int(11) 	  NOT NULL COMMENT '大题号',
    `sub_number`      varchar(32) NOT NULL COMMENT '小题号',
    `avg_score`       double      DEFAULT NULL COMMENT '平均分',
    `total_score`     double      DEFAULT NULL COMMENT '满分',
    `stdev`           double      DEFAULT NULL COMMENT '标准差',
    `score_rate`      double      DEFAULT NULL COMMENT '得分率',
    `full_score_rate` double      DEFAULT NULL COMMENT '满分率',
    `coefficient`     double      DEFAULT NULL COMMENT '差异系数',
    `difficulty`      double      DEFAULT NULL COMMENT '难度',
    `discrimination`  double      DEFAULT NULL COMMENT '区分度',
    `max_score`       double      DEFAULT NULL COMMENT '最大分数',
    `min_score`       double      DEFAULT NULL COMMENT '最小分数',
    `reality_count`   int(11)     DEFAULT NULL COMMENT '有效数量',
    `zero_count`      int(11)     DEFAULT NULL COMMENT '零分数量',
    `options`         text        DEFAULT NULL COMMENT '选项',
    `answer`          varchar(16) DEFAULT NULL COMMENT '答案',
    `full_count`      int(11)     DEFAULT NULL COMMENT '满分数量',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='小题统计表';


# Dump of table s_basic_subject
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject`;
CREATE TABLE `s_basic_subject`
(
    `id`                   int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`              int(11)     DEFAULT NULL COMMENT '考试ID',
    `subject_code`         varchar(32) DEFAULT NULL COMMENT '科目代码',
    `subject_name`         varchar(32) DEFAULT NULL COMMENT '科目名称',
    `absent_count`         int(11)     DEFAULT NULL COMMENT '缺考人数',
    `breach_count`         int(11)     DEFAULT NULL COMMENT '违纪人数',
    `pass_count`           int(11)     DEFAULT NULL COMMENT '及格人数',
    `pass_rate`            double      DEFAULT NULL COMMENT '及格率',
    `reality_count`        int(11)     DEFAULT NULL COMMENT '有效人数',
    `max_score`            double      DEFAULT NULL COMMENT '最高分',
    `min_score`            double      DEFAULT NULL COMMENT '最低分',
    `avg_score`            double      DEFAULT NULL COMMENT '平均分',
    `total_count`          int(11)     DEFAULT NULL COMMENT '总人数',
    `score_range`          text COMMENT '分数分布',
    `options`              text        DEFAULT NULL COMMENT '选项',
    `difficulity_level`    text        DEFAULT NULL COMMENT '难度分布',
    `discrimination_level` text        DEFAULT NULL COMMENT '区分度分布',
    `range_level`          text        DEFAULT NULL COMMENT '高低分段统计',
    `coefficient`          double      DEFAULT NULL COMMENT '差异系数',
    `difficulty`           double      DEFAULT NULL COMMENT '难度',
    `discrimination`       double      DEFAULT NULL COMMENT '区分度',
    `full_count`           int(11)     DEFAULT NULL COMMENT '满分人数',
    `question_count`       int(11)     DEFAULT NULL COMMENT '题目数量',
    `stdev`                double      DEFAULT NULL COMMENT '方差',
    `total_score`          double      DEFAULT NULL COMMENT '满分',
    `zero_count`           int(11)     DEFAULT NULL COMMENT '零分数量',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='科目统计表';


# Dump of table s_basic_subject_class
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject_class`;
CREATE TABLE `s_basic_subject_class`
(
    `id`                   int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`              int(11)     DEFAULT NULL COMMENT '考试ID',
    `subject_code`         varchar(32) DEFAULT NULL COMMENT '科目代码',
    `subject_name`         varchar(32) DEFAULT NULL COMMENT '科目名称',
    `class_name`           varchar(64) DEFAULT NULL COMMENT '班级名称',
    `excellent_count`      int(11)     DEFAULT NULL COMMENT '优秀人数',
    `excellent_rate`       double      DEFAULT NULL COMMENT '优秀率',
    `pass_count`           int(11)     DEFAULT NULL COMMENT '及格人数',
    `pass_rate`            double      DEFAULT NULL COMMENT '及格率',
    `max_score`            double      DEFAULT NULL COMMENT '最高分',
    `min_score`            double      DEFAULT NULL COMMENT '最低分',
    `avg_score`            double      DEFAULT NULL COMMENT '平均分',
    `coefficient`          double      DEFAULT NULL COMMENT '差异系数',
    `difficulty`           double      DEFAULT NULL COMMENT '难度',
    `score_range`          text        DEFAULT NULL COMMENT '分数分布',
    `stdev`                double      DEFAULT NULL COMMENT '方差',
    `difficulity_level`    text        DEFAULT NULL COMMENT '难度分布',
    `discrimination_level` text        DEFAULT NULL COMMENT '区分度分布',
    `range_level`          text        DEFAULT NULL COMMENT '高低分分布',
    `reality_count`        int(11)     DEFAULT NULL COMMENT '有效数量',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='班级统计表';


# Dump of table s_basic_subject_college
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject_college`;
CREATE TABLE `s_basic_subject_college`
(
    `id`              int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`         int(11)     DEFAULT NULL COMMENT '考试ID',
    `subject_code`    varchar(32) DEFAULT NULL COMMENT '科目代码',
    `subject_name`    varchar(32) DEFAULT NULL COMMENT '科目名称',
    `college_name`    varchar(64) DEFAULT NULL COMMENT '学院名称',
    `excellent_count` int(11)     DEFAULT NULL COMMENT '优秀人数',
    `excellent_rate`  double      DEFAULT NULL COMMENT '优秀率',
    `pass_count`      int(11)     DEFAULT NULL COMMENT '及格人数',
    `pass_rate`       double      DEFAULT NULL COMMENT '及格率',
    `max_score`       double      DEFAULT NULL COMMENT '最高分',
    `min_score`       double      DEFAULT NULL COMMENT '最低分',
    `avg_score`       double      DEFAULT NULL COMMENT '平均分',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='学院统计表';


# Dump of table s_basic_subject_teacher
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject_teacher`;
CREATE TABLE `s_basic_subject_teacher`
(
    `id`                 int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`            int(11)     DEFAULT NULL COMMENT '考试ID',
    `subject_code`       varchar(32) DEFAULT NULL COMMENT '科目代码',
    `subject_name`       varchar(32) DEFAULT NULL COMMENT '科目名称',
    `teacher_name`       varchar(64) DEFAULT NULL COMMENT '任课老师',
    `excellent_count`    int(11)     DEFAULT NULL COMMENT '优秀人数',
    `excellent_rate`     double      DEFAULT NULL COMMENT '优秀率',
    `pass_count`         int(11)     DEFAULT NULL COMMENT '及格人数',
    `pass_rate`          double      DEFAULT NULL COMMENT '及格率',
    `max_score`          double      DEFAULT NULL COMMENT '最高分',
    `min_score`          double      DEFAULT NULL COMMENT '最低分',
    `avg_score`          double      DEFAULT NULL COMMENT '平均分',
    `reality_count`      int(11)     DEFAULT NULL COMMENT '有效人数',
    `relative_avg_score` double      DEFAULT NULL COMMENT '平均相对分',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='任课老师统计表';


# Dump of table s_basic_subject_teacher_class
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_basic_subject_teacher_class`;
CREATE TABLE `s_basic_subject_teacher_class`
(
    `id`                 int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`            int(11)     DEFAULT NULL COMMENT '考试ID',
    `subject_code`       varchar(32) DEFAULT NULL COMMENT '科目代码',
    `subject_name`       varchar(32) DEFAULT NULL COMMENT '科目名称',
    `teacher_name`       varchar(64) DEFAULT NULL COMMENT '任课老师',
    `class_name`         varchar(64) DEFAULT NULL COMMENT '班级名称',
    `excellent_count`    int(11)     DEFAULT NULL COMMENT '优秀人数',
    `excellent_rate`     double      DEFAULT NULL COMMENT '优秀率',
    `pass_count`         int(11)     DEFAULT NULL COMMENT '及格人数',
    `pass_rate`          double      DEFAULT NULL COMMENT '及格率',
    `max_score`          double      DEFAULT NULL COMMENT '最高分',
    `min_score`          double      DEFAULT NULL COMMENT '最低分',
    `avg_score`          double      DEFAULT NULL COMMENT '平均分',
    `reality_count`      int(11)     DEFAULT NULL COMMENT '有效人数',
    `relative_avg_score` double      DEFAULT NULL COMMENT '平均相对分',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='任课老师班级统计表';


# Dump of table s_range_subject
# ------------------------------------------------------------

DROP TABLE IF EXISTS `s_range_subject`;
CREATE TABLE `s_range_subject`
(
    `id`            int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
    `exam_id`       int(11)     DEFAULT NULL COMMENT '考试ID',
    `subject_code`  varchar(32) DEFAULT NULL COMMENT '科目代码',
    `subject_name`  varchar(32) DEFAULT NULL COMMENT '科目名称',
    `subject_level` varchar(64) DEFAULT NULL COMMENT '科目层次',
    `totalCount`    int(11)     DEFAULT NULL COMMENT '总人数',
    `count0_49`     int(11)     DEFAULT NULL COMMENT '0-49',
    `count50_59`    int(11)     DEFAULT NULL COMMENT '50-59',
    `count60_69`    int(11)     DEFAULT NULL COMMENT '60-69',
    `count70_79`    int(11)     DEFAULT NULL COMMENT '70-79',
    `count80_89`    int(11)     DEFAULT NULL COMMENT '80-89',
    `count90_100`   int(11)     DEFAULT NULL COMMENT '90-100',
    `lt60`          int(11)     DEFAULT NULL COMMENT '<60',
    `mte60`         int(11)     DEFAULT NULL COMMENT '>=60',
    `percent0_49`   double  NOT NULL COMMENT '0-49占比',
    `percent50_59`  double  NOT NULL COMMENT '50-59占比',
    `percent60_69`  double  NOT NULL COMMENT '60-69占比',
    `percent70_79`  double  NOT NULL COMMENT '70-79占比',
    `percent80_89`  double  NOT NULL COMMENT '80-89占比',
    `percent90_100` double  NOT NULL COMMENT '90-100占比',
    `percent_lt60`  double  NOT NULL COMMENT '<60占比',
    `percent_mte60` double  NOT NULL COMMENT '>=60占比',
    PRIMARY KEY (`id`),
    KEY `index1` (`exam_id`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='分段统计表';
  

# Dump of table b_school
# ------------------------------------------------------------

DROP TABLE IF EXISTS `eb_import_query`;

CREATE TABLE `eb_import_query` 
(
  `id` 			int(11) 	NOT NULL AUTO_INCREMENT COMMENT '主键',
  `create_time` datetime    NOT NULL COMMENT '创建时间',
  `description` text 		DEFAULT NULL COMMENT '描述',
  `exam_id` 	int(11) 	NOT NULL COMMENT '考试ID',
  `type` 		varchar(16) NOT NULL COMMENT '类型',
  `user_id` 	int(11)  	NOT NULL COMMENT '用户ID',
  PRIMARY KEY (`id`),
  KEY `index1` (`exam_id`, `user_id`, `type`)
)  ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='导入查询表';
